#[名称]jude2ruby
#[機能]JUDE/Communityが出力するJavaのスケルトンファイルをRubyのスケルトンファイルに変換
#[使用方法]Javaスケルトンファイルのあるフォルダにjude2ruby.rbを置き実行する
#
#[改訂履歴]
#Date       Ver  Name     Comment
#2007/10/09 0.00 hantani  new

require 'logger'
require 'kconv'

class MyProperty
  attr_accessor :access, :type, :name

  TypePrivate = "private"
  TypePublic = "public"

  def initialize(access,type,name)
    @access = access
    @type = type
    @name = name
  end
end
class MyMethod
  attr_accessor :access, :type, :name,:argument

  TypePrivate = "private"
  TypePublic = "public"

  def initialize(access,type,name)
    @access = access
    @type = type
    @name = name
    @argument=[]
  end
  def addArgument(arg)
    @argument << arg
  end
end

class RubyClass
  attr_accessor :className, :property, :method, :extends
  def initialize
    @className=""
    @property=[]
    @method=[]
    @extends=[]
  end
end
  def analyze(line)
    if /class\s+(\S+).*\{$/=~ line #class name
      @rubyClass.className = $1
      @logger.debug "class name #{$1}"
      if /class\s+(\S+)\s*extends\s+([^\{]+)\{$/=~ line #extends name
        cls = $2.split(",")
        cls.each {|c| @rubyClass.extends << c.strip}
      end
    end
    if /\s*(\S+)\s+(\S+)\s+(\S+)\s*;/ =~ line #propaty
      prp=MyProperty.new($1,$2,$3)
      @logger.debug "MyProperty #{$1},#{$2},#{$3}"
      @rubyClass.property << prp
    end    
    if /\s*(\S+)\s+(\S+)\s+(\S+)\s*\(([^\)]*)\)\s+\{/ =~ line #method
      met = MyMethod.new($1,$2,$3)
      @logger.debug "MyMethod #{$1},#{$2},#{$3}"
      if $4 != "" then
        prm = $4.split(",")
        prm.each {|c|
          d = c.split(" ")
          @logger.debug "arg #{d[1].strip}"
          met.argument << d[1].strip
        }
      end
      @rubyClass.method << met
    end
  end
class Jude2ruby
  def initialize(logger = nil)
    @logger = logger || Logger.new(STDERR)
#    @logger.level = Logger::DEBUG
    @logger.level = Logger::WARN
  end
  attr_reader :logger
  def readfile(file)
    @rubyClass=RubyClass.new
    f=File.open(file)
    while line = f.gets
      if line then
        line = Kconv.toutf8(line)
        analyze(line)
      end
    end
    f.close
  end
  def writefile(file)
    wrfile = File.dirname(file) + "/" + File.basename(file, ".*") + ".rb"
    @logger.debug "w:#{wrfile}"
    f = File.open(wrfile,"w")
    f.puts(convert)
    f.close
  end
  def convert
    
    c = @rubyClass.className

    if c[0..0] != c[0..0].upcase then
      msg = "#{@rubyClass.className} :class/module name must be CONSTANT"
      @logger.error msg
      return msg
    end

    s="\nclass #{c}"
    
    if @rubyClass.extends.size > 0 then
      e = @rubyClass.extends[0]
      if e[0..0] != e[0..0].upcase then
        msg = "#{e} :class/module name must be CONSTANT"
        @logger.error msg
        return msg
      end
      s = s + " < #{@rubyClass.extends[0]}"
      s = "require '#{@rubyClass.extends[0]}'\n" + s
    end
    s = s + "\n"
    pubPrp = ""
    prvPrp = ""
    @rubyClass.property.each {|prp|
      if /(\S+)\[\]/=~ prp.type then
        s = "require '#{$1}'\n" + s
      end
      
      if prp.access == MyProperty::TypePrivate then
        prvPrp = prvPrp + "\t@#{prp.name}\n"
      end
      if prp.access == MyProperty::TypePublic then
        if pubPrp == "" then
          pubPrp = pubPrp + ":#{prp.name}"
        else
          pubPrp = pubPrp + ",:#{prp.name}"
        end
        prvPrp = prvPrp + "\t@#{prp.name}\n"
      end
    }
    if pubPrp != "" then
      s = s + "\tattr_accessor " + pubPrp + "\n"
    end
    if prvPrp != "" then
      s = s + prvPrp
    end
    s = s + "\n"
    pubMet = ""
    prvMet = ""
    @rubyClass.method.each {|met|
      m = ""
      met.argument.each{|arg|
        if m == "" then
          m = m + arg
        else
          m = m + "," + arg
        end
      }
      if not m then
        m = ""
      end
      m = "\tdef #{met.name}(" + m + ")\n\tend\n"
      @logger.debug "met.access #{met.access}" 
      @logger.debug "met.access #{MyMethod::TypePrivate}"
      @logger.debug "met.access #{MyMethod::TypePublic}"
      if met.access == MyMethod::TypePrivate then
        prvMet = prvMet + m
        @logger.debug "prvMet #{m}"
      end
      if met.access == MyMethod::TypePublic then
        pubMet = pubMet + m
        @logger.debug "pubMet #{m}"
      end
    }
    s = s + pubMet
    if prvMet != "" then
      s = s + "\tprivate\n"
      s = s + prvMet
    end
    s = s + "end"
  end
  def main(file)
    readfile(file)
    writefile(file)
  end
end


j2r=Jude2ruby.new
Dir.glob("**/*.java") do |file|
  j2r.main(file)
end
